/*RT_OTS compute diffuse fields due to H, He atoms, ion, triplets, metal recombination,
 * called by ConvIonizeOpacityDo  */
/*RT_OTS_AddLine add local destruction of lines to ots field */
/*RT_OTS_AddCont add local destruction of continuum to ots field */
/*RT_OTS_Update sum flux, otscon, otslin, ConInterOut, outlin, to form SummeDif, SummedCon SummedOcc */
/*RT_OTS_Zero - zero out some vectors - 
 * this is only called when code initialized by ContSetIntensity */
/*RT_OTS_ChkSum sanity check confirms summed continua reflect contents of individuals */
#include "cddefines.h"
#include "nhe3lvl.h"
#include "taulines.h"
#include "nhe1lvl.h"
#include "iso.h"
#include "opacity.h"
#include "he3n.h"
#include "phe1lv.h"
#include "phycon.h"
#include "heots.h"
#include "rfield.h"
#include "converge.h"
#include "nhe1.h"
#include "he3lines.h"
#include "d10830.h"
#include "iphe1l.h"
#include "ip626.h"
#include "rt.h"
#include "he3as.h"
#include "sphere.h"
#include "ionfracs.h"
#include "pop371.h"
#include "heavy.h"
#include "abundances.h"
#include "pbowen.h"
#include "ionrange.h"
#include "radius.h"
#include "trace.h"

/* =================================================================== */

void RT_OTS(void)
{
	long int i, 
		ipla,
	  j, 
	  nelem,
	  n;
	float 
		difflya,
		esc,
	  escfac, 
	  ots,
	  outward;

	/* the Bowen HeII yield */
#	define BOWEN 0.3f
	long int ipHi, 
	  ipLo, 
	  ipZ;

	float bwnfac ,
	  ots660 ,
	  otsnew;

	float totline,
	  otsfac;
	float otshere; 

#	ifdef DEBUG_FUN
	fputs( "<+>RT_OTS()\n", debug_fp );
#	endif

	/**************************************************************************
	 *
	 * the bowen HeII - OIII fluorescense problem
	 *
	 **************************************************************************/

	ipZ = 1;
	if( abundances.lgElmtOn[ipZ] )
	{
		/* conversion per unit atom to OIII, at end of sub we divide by it,
		 * to fix lines back to proper esc/dest probs */
		bwnfac = BOWEN * (1.f - EmisLines[ipHYDROGEN][ipZ][ipH2p][ipH1s].Pesc );

		/* escaping part that will be added to the outward beam in metdif,
		 * this uses ipLnEsc instead of ipLnDesp */
		heots.esc660 = (float)(EmisLines[ipHYDROGEN][ipZ][ipH2p][ipH1s].Aul*
		  EmisLines[ipHYDROGEN][ipZ][ipH2p][ipH1s].PopHi*xIonFracs[ipZ][ipZ+2]*
		  EmisLines[ipHYDROGEN][ipZ][ipH2p][ipH1s].Pesc *BOWEN*2.0 * 0.8 );

		/* the last factor accounts for fact that two photons are produced,
		 * and the branching ratio */
		ots660 = (float)(EmisLines[ipHYDROGEN][ipZ][ipH2p][ipH1s].Aul*
		  EmisLines[ipHYDROGEN][ipZ][ipH2p][ipH1s].PopHi*xIonFracs[ipZ][ipZ+2]*
		  EmisLines[ipHYDROGEN][ipZ][ipH2p][ipH1s].Pdest *BOWEN*2.0 * 0.8 );

		/* now add this to the ots field */
		RT_OTS_AddLine(ots660 , pbowen.ip660 );

		/* decrease the destruction prob by the amount we will add elsewhere,
		 * ok since dest probs updated on every iteration*/
		EmisLines[ipHYDROGEN][ipZ][ipH2p][ipH1s].Pdest *= bwnfac;
	}

	else
	{
		bwnfac = 1.;
	}

	/* make ots fields due to hydrogen lines and continua */

	/* loop over all elements */
	for( ipZ=0; ipZ < LIMELM; ipZ++ )
	{
		/* do if this stage exists */
		if( (IonRange.IonHigh[ipZ] == ipZ+1)  )
		{

			/**************************************************************************
			 *
			 * generate hydrogenic line ots rates 
			 *
			 **************************************************************************/

			/* now loop over all possible levels, but cannot include two photon
			 * since there is no pointer to this continuum */
			for( ipHi=ipH2p; ipHi < iso.nLevels[ipHYDROGEN][ipZ]; ipHi++ )
			{
				for( ipLo=ipH1s; ipLo < ipHi; ipLo++ )
				{
					/* this signifies a fake line */
					if( EmisLines[ipHYDROGEN][ipZ][ipHi][ipLo].ipCont< 1 )
						continue;
					/* ots rates, the destp prob was set in hydropesc */
					EmisLines[ipHYDROGEN][ipZ][ipHi][ipLo].ots = (float)(
						EmisLines[ipHYDROGEN][ipZ][ipHi][ipLo].Aul*
						EmisLines[ipHYDROGEN][ipZ][ipHi][ipLo].PopHi*xIonFracs[ipZ][ipZ+2]*
						EmisLines[ipHYDROGEN][ipZ][ipHi][ipLo].Pdest);

					assert( EmisLines[ipHYDROGEN][ipZ][ipHi][ipLo].ots >= 0. );
					/* option to kill lyalpha ots rates 
					if( ipZ==0 && ipHi==ipH2p && ipLo==ipH1s )
					{
						EmisLines[ipHYDROGEN][ipZ][ipHi][ipLo].ots = 0.;
					}*/

					/* finally dump the ots rate into the stack */
					RT_OTS_AddLine(EmisLines[ipHYDROGEN][ipZ][ipHi][ipLo].ots,
						EmisLines[ipHYDROGEN][ipZ][ipHi][ipLo].ipCont );
				}
			}
			{
				/* debugging code for Lya problems */
				/*@-redef@*/
				enum {DEBUG=FALSE};
				/*@+redef@*/
				if( DEBUG )
				{
					if( ipZ==7  && nzone > 44 ) 
					{
#						if 0
					  fprintf(ioQQQ,"%li blr89bugg hlyaots\t%g\t%g\t%g\t%g\t%g\t%g\t%g\n",
							conv.nTotalIoniz, 
							EmisLines[ipHYDROGEN][ipZ][ipH2p][ipH1s].TauIn ,
							EmisLines[ipHYDROGEN][ipZ][ipH2p][ipH1s].TauTot,
							EmisLines[ipHYDROGEN][ipZ][ipH2p][ipH1s].Pdest,
							EmisLines[ipHYDROGEN][ipZ][ipH2p][ipH1s].ots , 
							opac.opac[EmisLines[ipHYDROGEN][ipZ][ipH2p][ipH1s].ipCont-1] ,
							rfield.otslinNew[EmisLines[ipHYDROGEN][ipZ][ipH2p][ipH1s].ipCont-1] ,
							EmisLines[ipHYDROGEN][ipZ][ipH2p][ipH1s].PopHi*xIonFracs[ipZ][ipZ+2]
							);
#						endif
						fprintf(ioQQQ,"%li hlyaots\t%g\t%g\t%g\n",
							nzone, xIonFracs[ipZ][ipZ+2] ,
							xIonFracs[ipSILICON][12],
							xIonFracs[ipIRON][16]);
					}
				}
			}

			/**************************************************************************
			 *
			 * ots recombination bound-free b-f continua continuum
			 *
			 **************************************************************************/

			/* put in OTS continuum */
			for( n=ipH1s; n < iso.nLevels[ipHYDROGEN][ipZ]; n++ )
			{
				otsnew = (float)(iso.RadRecomb[ipHYDROGEN][ipZ][n][ipRecRad]*
					(1. - iso.RadRecomb[ipHYDROGEN][ipZ][n][ipRecEsc])*phycon.eden*
					xIonFracs[ipZ][ipZ+2]);
				assert( otsnew >= 0. );

				/*otsnew = (rfield.otscon[iso.ipIsoLevNIonCon[ipHYDROGEN][ipZ][n]-1]*
					(1.f - damp) + 
					damp*iso.RadRecomb[ipHYDROGEN][ipZ][n][ipRecRad]*
					(1.f - iso.RadRecomb[ipHYDROGEN][ipZ][n][ipRecEsc])*phycon.eden*
					xIonFracs[ipZ][ipZ+2]/
					opac.opac[iso.ipIsoLevNIonCon[ipHYDROGEN][ipZ][n]-1])/2.f;*/

				/* continuum energy index used in this routine is decremented by one there */
				RT_OTS_AddCont(otsnew,iso.ipIsoLevNIonCon[ipHYDROGEN][ipZ][n]);
				/* debugging code for rec continua */
				{
					/*@-redef@*/
					enum {DEBUG=FALSE};
					/*@+redef@*/
					if( DEBUG )
					{
						if( nzone > 234 && ipZ==0 && n==0) 
						{
							fprintf(ioQQQ,"hotsdebugg %li\t%.4e\th con ots\t%g\t%g\t%g\t%g\t%g\t%g\n",
								nzone, 
								radius.depth ,
								EmisLines[ipHYDROGEN][ipHYDROGEN][2][0].PopHi*xIonFracs[ipHYDROGEN][2],
								otsnew,
								iso.RadRecomb[ipHYDROGEN][ipZ][n][ipRecEsc] , 
								opac.opac[iso.ipIsoLevNIonCon[ipHYDROGEN][ipZ][n]-1] ,
								rfield.otsconNew[iso.ipIsoLevNIonCon[ipHYDROGEN][ipZ][n]-1] ,
								opac.e2TauAbs[iso.ipIsoLevNIonCon[ipHYDROGEN][ipZ][n]-1] );
						}
					}
				}
			}
		}
	}
	/* more debugging code for rec continua */
	{
		/*@-redef@*/
		enum {DEBUG=FALSE};
		/*@+redef@*/
		if( DEBUG )
		{
			ipZ = 0;
			fprintf(ioQQQ,"hotsdebugg %li \t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\n",
				nzone, 
				rfield.otsconNew[iso.ipIsoLevNIonCon[ipHYDROGEN][ipZ][0]-1],
				rfield.otsconNew[iso.ipIsoLevNIonCon[ipHYDROGEN][ipZ][1]-1],
				rfield.otsconNew[iso.ipIsoLevNIonCon[ipHYDROGEN][ipZ][3]-1],
				rfield.otsconNew[iso.ipIsoLevNIonCon[ipHYDROGEN][ipZ][4]-1],
				rfield.otsconNew[iso.ipIsoLevNIonCon[ipHYDROGEN][ipZ][5]-1],
				rfield.otsconNew[iso.ipIsoLevNIonCon[ipHYDROGEN][ipZ][6]-1],
				       opac.opac[iso.ipIsoLevNIonCon[ipHYDROGEN][ipZ][6]-1]);
		}
	}

	ipZ = 1;
	if( abundances.lgElmtOn[ipZ] && bwnfac > 0. )
	{
		/* increase the destruction prob by the amount we wdecreased it above */
		EmisLines[ipHYDROGEN][ipZ][ipH2p][ipH1s].Pdest /= bwnfac;
	}

	if( trace.lgTrace )
	{
		fprintf(ioQQQ,"     RT_OTS Pdest %.2e ots rate %.2e in otslinNew %.2e con opac %.2e\n",
			EmisLines[ipHYDROGEN][ipHYDROGEN][ipH2p][ipH1s].Pdest , EmisLines[ipHYDROGEN][ipHYDROGEN][ipH2p][ipH1s].ots , 
			rfield.otslinNew[EmisLines[ipHYDROGEN][ipHYDROGEN][ipH2p][ipH1s].ipCont-1]  ,
			opac.opac[EmisLines[ipHYDROGEN][ipHYDROGEN][ipH2p][ipH1s].ipCont-1]
			);
	}

	/* fixit(); replace this with he iso continua in loop above */
	/* He I recombination continuum */
	for( n=0; n < 6; n++ )
	{
		otshere = (float)(phe1lv.he1rec[0][n]*(1. - phe1lv.he1rec[ipRecEsc][n])*
		  phycon.eden*xIonFracs[ipHELIUM][2]) ;
#		if 0
		if( n==0 )
			fprintf(ioQQQ,"he1 otsconnnn\t%.2e\t%.2e\t%.2e\t%.2e\n",
			otshere, 
			phe1lv.he1rec[0][n],
			phe1lv.he1rec[ipRecEsc][n] ,
			opac.opac[nhe1Com.nhe1[n]-1]
			);
#		endif
		assert( otshere>= 0. );
		RT_OTS_AddCont(otshere,nhe1Com.nhe1[n]);
	}

	/* HeI recombination continua at 1.81 Ryd */
	heots.esc504 = (float)(phe1lv.he1rec[0][0]*phe1lv.he1rec[ipRecEsc][0]*phycon.eden*
	  xIonFracs[ipHELIUM][2]);

	/* HeI Ly-alpha He I ly a
	 * escaping part of HeI Ly-alpha lya ly a */
	heots.esc584 = 
		xIonFracs[ipHELIUM][2]*phe1lv.he12p*phe1lv.he1mis[0][1]*
		opac.ExpmTau[iphe1lCom.iphe1l[0][1]-1];

	/* do not want to add much outward HeI Lya since it is so important
	 * use inward looking optical depth to decide outward flux */
	otshere = (float)(phe1lv.he12p*xIonFracs[ipHELIUM][2]*phe1lv.he1dst[0][1] + xIonFracs[ipHELIUM][2]*phe1lv.he12p*
	  phe1lv.he1mis[0][1]*(1.f - opac.ExpmTau[iphe1lCom.iphe1l[0][1]-1]));

	if( otshere < 0. )
	{
		fprintf( ioQQQ, " RT_OTS found negative OTS He1 alhpa,%4i val=%10.1e\n", 
		  1, otshere );
		ShowMe();
		puts( "[Stop in hediff]" );
		cdEXIT(1);
	}
	RT_OTS_AddLine(otshere,iphe1lCom.iphe1l[0][1] );

	/* all other lines of HeI - tests show this is very small effect */
	for( j=2; j < 7; j++ )
	{
		for( i=0; i <= (j - 1); i++ )
		{
			otshere = phe1lv.he1n[j]*xIonFracs[ipHELIUM][2]*phe1lv.he1dst[i][j];
			if( otshere < 0. )
			{
				fprintf( ioQQQ, " RT_OTS found negative OTS He1l%4ld%4ld val=%10.1e\n", 
				  j, i, otshere );
				ShowMe();
				puts( "[Stop in hediff]" );
				cdEXIT(1);
			}
			RT_OTS_AddLine(otshere,iphe1lCom.iphe1l[i][j]);
		}
	}

	/* HeI 2tripS to ground - one photon decay
	 * this is a major source of ionization for parishii */
	totline = xIonFracs[ipHELIUM][2]*he3nCom.he3n[0]*he3as.a2sg;
	if( sphere.lgSphere )
	{
		/* all goes outward or ots */
		escfac = 0.;
	}
	else
	{
		escfac = 0.5f*opac.ExpmTau[ip626.ipHe23sGrnd-1];
	}

	/* number of phots escaping from illuminated face of cloud */
	heots.esc626 = (float)(totline*escfac);

	/* otsfac = 1. - ExpZone(ipHe23sGrnd)
	 * when optically thin, this goes to zero, when thick, to 100% */
	otsfac = 1.f - 2.f*escfac;

	/* >>chng 96 mar 07, above had been 1-escfac, so 50% ots when thin to cont */
	RT_OTS_AddLine(totline*otsfac,ip626.ipHe23sGrnd);

	/* following is zero when thick to cont, 0.5 when thin */
	outward = MAX2(0.f,1.f-otsfac-escfac);

	/* outward escaping line */
	heots.out626 = (float)(totline*outward);

	/* HeI 10830 OTS */
	RT_OTS_AddLine(he3lines.p2p*d10830.DR10830,he3lines.ipHe3l[0]);

	/*>>chng 99 feb 17 this code had been in metdif, moved here in cleanup */
	/* all locally destroyed two photon continua of hydrogenic H, HeI, HeII */
	/* >>chng 00 dec 27, remove these and replace with locally destroyed ots continuum,
	 * rfield.ConOTS_local_photons */
#	if 0
	if( hhe2phtots.H12 phtOTS < 0. )
	{
		fprintf( ioQQQ, " RT_OTS found negative OTS CON, H12phtOTS val=%10.1e\n", 
		  hhe2phtots.H12 phtOTS );
		ShowMe();
		puts( "[Stop in RT_OTS]" );
		cdEXIT(1);
	}
	RT_OTS_AddCont(hhe2phtots.H12 phtOTS,hhe2phtots.ipH12pht);
	{
		/*@-redef@*/
		enum {DEBUG=FALSE};
		/*@+redef@*/
		if( DEBUG )
		{
			fprintf(ioQQQ," RT_OTS expmtau %e %e\n" , 
				opac.ExpmTau[EmisLines[ipHYDROGEN][ipHYDROGEN][ipH2p][ipH1s].ipCont - 30 ],
				hhe2phtots.H12 phtOTS);
		}
	}

	if( hhe2phtots.He12 phtOTS < 0. )
	{
		fprintf( ioQQQ, " RT_OTS found negative OTS CON, He12 phtOTS val=%10.1e\n", 
		  hhe2phtots.He12 phtOTS );
		ShowMe();
		puts( "[Stop in RT_OTS]" );
		cdEXIT(1);
	}
	RT_OTS_AddCont(hhe2phtots.He12 phtOTS,hhe2phtots.ipHe12pht);

	if( hhe2phtots.He22 phtOTS < 0. )
	{
		fprintf( ioQQQ, " RT_OTS found negative OTS CON, He22 phtOTS val=%10.1e\n", 
		  hhe2phtots.He22 phtOTS );
		ShowMe();
		puts( "[Stop in RT_OTS]" );
		cdEXIT(1);
	}
	RT_OTS_AddCont(hhe2phtots.He22 phtOTS,hhe2phtots.ipHe22pht);
#	endif

	/* add recombination continua for elements heavier than He */
	/* nelem = 2 is Li */
	for( nelem=2; nelem < LIMELM; nelem++ )
	{
		/* do not include hydrogenic species in following */
		for( i=0; i < nelem; i++ )
		{
			if( xIonFracs[nelem][i+2] > 0. )
			{
				/* now do the recombination Lya */
				ipla = Heavy.ipLyHeavy[i][nelem];
				esc = opac.ExpmTau[ipla-1];
				/* xLyaHeavy is set to a fraction of total rad rec in MakeRecomb, includes eden */
				difflya = Heavy.xLyaHeavy[i][nelem]*xIonFracs[nelem][i+2];
				/* >>chng 00 dec 22, from MIN2 to MAX2, MIN2 had effect of always
				 * setting the ots rates to zero */
				ots = difflya*MAX2(0.f,1.f-esc);
				/* ots *= 0.;testcode();del this line */
				assert( ots >= 0.);
				RT_OTS_AddLine(ots,ipla);

				/* now do the recombination balmer lines */
				ipla = Heavy.ipBalHeavy[i][nelem];
				esc = opac.ExpmTau[ipla-1];
				/* xLyaHeavy is set to a fraction of total rad rec in MakeRecomb, includes eden */
				difflya = Heavy.xLyaHeavy[i][nelem]*xIonFracs[nelem][i+2];
				/* >>chng 00 dec 22, from MIN2 to MAX2, MIN2 had effect of always
				 * setting the ots rates to zero */
				ots = difflya*MAX2(0.f,1.f-esc);
				assert( ots >= 0.);
				RT_OTS_AddLine(ots,ipla);
			}
		}
	}

	if( FeII.lgFeIION )
	{
		/* do OTS and outward parts of FeII lines, if large atom is turned on */
		Fe2RTOTS();
	}

#	ifdef DEBUG_FUN
	fputs( " <->RT_OTS()\n", debug_fp );
#	endif

	return;
}

/* =================================================================== */

void RT_OTS_AddLine(float ots, 
	/* pointer on the f scale */
  long int ip )
{

#	ifdef DEBUG_FUN
	fputs( "<+>RT_OTS_AddLine()\n", debug_fp );
#	endif

	/* add ots due to line destruction to radiation field */

	/* return if outside bounds of this continuum source, ip > rfield.nflux
	 * first case ip==0 happens when called with dummy line */
	if( ip==0 || ip > rfield.nflux )
	{ 
#		ifdef DEBUG_FUN
		fputs( " <->RT_OTS_AddLine()\n", debug_fp );
#		endif
		return;
	}

	/*the local ots rate must be non-negative */
	assert( ots >= 0. );
	/* continuum pointer must be positive */
	assert( ip > 0 );

	/* add locally destroyed flux of photons to line OTS array */
	if( opac.opac[ip-1] > 0. )
	{
		rfield.otslinNew[ip-1] += (float)(ots/opac.opac[ip-1]);
	}

#	ifdef DEBUG_FUN
	fputs( " <->RT_OTS_AddLine()\n", debug_fp );
#	endif
	return;
}

/* =================================================================== */

/*add local destruction of continuum to ots field */
void RT_OTS_AddCont(
					/* the ots rate itself */
					float ots, 
					/* pointer to continuum cell for ots, on f scale */
					long int ip)
{

#	ifdef DEBUG_FUN
	fputs( "<+>RT_OTS_AddCont()\n", debug_fp );
#	endif

	/* 
	 * routine called to add ots due to continuum destruction to
	 * radiation field
	 */
	 
	/* check if outside bounds of this continuum source */
	if( ip > rfield.nflux )
	{ 
#		ifdef DEBUG_FUN
		fputs( " <->RT_OTS_AddCont()\n", debug_fp );
#		endif
		return;
	}

	assert( ip> 0 );

	assert( ots >= 0. );

	assert( ip <= rfield.nupper );

	/* add locally destroyed flux of photons to continuum OTS array */
	if( opac.opac[ip-1] > 0. )
	{
		rfield.otsconNew[ip-1] += (float)(ots/opac.opac[ip-1]);
	}

#	ifdef DEBUG_FUN
	fputs( " <->RT_OTS_AddCont()\n", debug_fp );
#	endif
	return;
}

/* =================================================================== */

void RT_OTS_Update(
	/* summed ots rates */
	double *SumOTS , 
	/* array index for constituent that changed the most */
	long int * ipOTSchange, 
	/* limit on how large a change to allow, 0 for no limit */
	double BigFrac)
{
	long int i;

	/* when setting new to sum of current and old, this is the fraction of old to take*/
	double FracOld , FracNew ,
		changeOTS ,
		chng,
		chngBig,
		OTSOld ,
		OTSNew,
		BigOTSOld,
		BigOTSNew,
		OpacRatio ;

#	ifdef DEBUG_FUN
	fputs( "<+>RT_OTS_Update()\n", debug_fp );
#	endif

	*SumOTS = 0.;

	OTSOld = 0.;
	OTSNew = 0.;
	BigOTSOld = 0.;
	BigOTSNew = 0.;
	/* find summed ots rates, old and new */
	for( i=0; i < rfield.nflux; i++ )
	{
		OTSOld += rfield.otscon[i] + rfield.otslin[i];
		OTSNew += rfield.otsconNew[i] + rfield.otslinNew[i];
		if( BigOTSNew < rfield.otsconNew[i] + rfield.otslinNew[i] )
		{
			BigOTSOld = rfield.otscon[i] + rfield.otslin[i];
			BigOTSNew = rfield.otsconNew[i] + rfield.otslinNew[i];
		}
	}
	
	/* we now have old and new rates, what is the ratio, and by how much will
	 * we allow this to change? */
	if( BigFrac == 0. || conv.lgSearch || OTSOld < SMALLFLOAT )
	{
		/* this branch, want a clear update of ots rates, either because
		 * requested with BigFrac == 0, or we are in search phase */
		FracOld = 0.;
	}
	else
	{
		chng = fabs(1. - OTSNew/MAX2(SMALLFLOAT,OTSOld) );
		if( BigOTSOld > SMALLFLOAT )
		{
			chngBig = fabs(1. - BigOTSNew/BigOTSOld );
		}
		else
		{
			chngBig = 0.;
		}
		chng = MAX2( chng , chngBig );

		if( chng < BigFrac )
		{
			/* this branch, old and new do not differ by much, so use new */
			FracOld = 0.;
		}
		else
		{
			/* this branch, too large a difference between old and new, cap it to BigFrac */
			FracOld = (1. - BigFrac / chng);
			FracOld = MIN2( 0.25 , FracOld );
		}
	}

	/* fraction old and new ots rates */
	FracNew = 1. - FracOld ;

	/* remember largest change in ots rates */
	changeOTS = 0.;
	*ipOTSchange = -100000;
	/*fprintf(ioQQQ," sumcontinuum zone%li 1168=%e\n", nzone,rfield.otslin[1167]);*/

	for( i=0; i < rfield.nflux; i++ )
	{
		/* >>chng 01 feb 01, define inverse opacity in safe manner */
		double CurrentInverseOpacity = 1./MAX2( SMALLDOUBLE , opac.opac[i] ) ;

		/* remember which energy had largest change in ots rates */
		if( fabs( rfield.otscon[i]+rfield.otslin[i]-rfield.otsconNew[i]-rfield.otslinNew[i])> changeOTS)
		{
			*ipOTSchange = i;

			changeOTS = 
			  fabs( rfield.otscon[i]+rfield.otslin[i]-rfield.otsconNew[i]-rfield.otslinNew[i]);
		}
		/* ots rates were evaluated with previous opacities, now have different
		 * opacities since OpacityAddTotal just called.  Rescale OTS rates to current opacity.
		 * tests with pure h, he constant temperature clouds shows this decreases*/
		/* >>chng 99 apr 9, rescale ots */
		OpacRatio = opac.OldOpacSave[i]*CurrentInverseOpacity;
		OpacRatio = 1.;

		/* >>chng 01 apr 10, only change ots with means if FracOld not zero,
		 * this is to avoid taking means when this routine called by StartEndIter 
		 * to reset sums of rates */
		if( BigFrac > 0. && conv.nTotalIoniz > 0 )
		{
			/* the New vectors are the ones updated by the AddOTS routines,
			 * and have the current OTS rates.  The otscon and otslin vectors
			 * have the rates from the previous refresh of the New vectors*/
			/* here is the refresh.  all were initially set to zero in call to 
			 * RT_OTS_Zero (below) from zero */
			rfield.otscon[i] = (float)((rfield.otscon[i]*FracOld +rfield.otsconNew[i]*FracNew)*OpacRatio);

			/* this is local ots continuum created by destroyed diffuse continua, 
			 * currently only two-photon */
			/* >>chng 00 dec 27, define this continuum and do not count 2-photon in old var */
			rfield.ConOTS_local_OTS_rate[i] = (float)((double)rfield.ConOTS_local_photons[i]*CurrentInverseOpacity);

			/* current ots will be average of old and new */
			rfield.otslin[i] = (float)((rfield.otslin[i]*FracOld + rfield.otslinNew[i]*FracNew)*OpacRatio);
		}

		/* zero out the new otscon vector*/
		rfield.otsconNew[i] = 0.;

		/* zero out the new otslin vector*/
		rfield.otslinNew[i] = 0.;

		/* remember sum of ots rates for convergence criteria */
		*SumOTS += (rfield.otscon[i] + rfield.otslin[i])*opac.opac[i];

		/* >>chng 00 dec 27, include ConOTS_local_OTS_rate */
		/* >>chng 01 jul 04, add *rfield.lgOutOnly logic */
		rfield.SummedDif[i] = rfield.otscon[i] + rfield.otslin[i] + 
		  rfield.ConInterOut[i]*rfield.lgOutOnly + rfield.outlin[i] +
		  rfield.ConOTS_local_OTS_rate[i];

		rfield.SummedCon[i] = rfield.flux[i] + rfield.SummedDif[i];
		rfield.SummedOcc[i] = rfield.SummedCon[i]*rfield.convoc[i];
	}

	{
		/* following should be set true to print strongest ots contributors */
		/*@-redef@*/
		enum {DEBUG=FALSE};
		/*@+redef@*/
		if( DEBUG && (nzone>0)/**/ )
		{
			double BigOTS;
			int ipOTS=0;
			BigOTS = -1.;
			/* find the biggest ots contributor */
			/*for( i=iso.ipIsoLevNIonCon[ipHYDROGEN][0][0]; i < rfield.nflux; i++ )*/
			for( i=0; i < rfield.nflux; i++ )
			{
				if( (rfield.otscon[i] + rfield.otslin[i])*opac.opac[i] > BigOTS )
				{
					BigOTS = (rfield.otscon[i] + rfield.otslin[i])*opac.opac[i];
					ipOTS = (int)i;
				}
			}
			fprintf(ioQQQ,
				" sumcontinuunots zone %li SumOTS %.2e %.2eRyd opc:%.2e OTS%.2e lin%.2e con:%.2e %s %s cell%i FracNew %.2f \n",
				nzone,
				*SumOTS,
				rfield.anu[ipOTS] , 
				opac.opac[ipOTS],
				BigOTS ,
				rfield.otslin[ipOTS],
				rfield.otscon[ipOTS] , 
				/* line	label */
				rfield.chLineLabel[ipOTS] ,
				/* cont label*/
				rfield.chContLabel[ipOTS] ,
				ipOTS,
				FracNew);
		}
	}

#	ifdef DEBUG_FUN
	fputs( " <->RT_OTS_Update()\n", debug_fp );
#	endif
	return;
}

/* =================================================================== */

/*RT_OTS_Zero - zero out some vectors - 
 * this is only called when code initialized by ContSetIntensity */
void RT_OTS_Zero( void )
{
	long int i;

#	ifdef DEBUG_FUN
	fputs( "<+>RT_OTS_Zero()\n", debug_fp );
#	endif

	/* this loop goes up to nflux itself (<=) since the highest cell
	 * will be used to pass unity through the code to verify integrations */
	for( i=0; i <= rfield.nflux; i++ )
	{
		rfield.SummedDif[i] = 0.;
		/* the four main ots vectors */
		rfield.otscon[i] = 0.;
		rfield.otslin[i] = 0.;
		rfield.otslinNew[i] = 0.;
		rfield.otsconNew[i] = 0.;

		rfield.ConInterOut[i] = 0.;
		rfield.outlin[i] = 0.;
		rfield.SummedDif[i] = 0.;
		/* "zero" for the summed con will be just the incident radiation field */
		rfield.SummedCon[i] = rfield.flux[i];
		rfield.SummedOcc[i] = rfield.SummedCon[i]*rfield.convoc[i];
		rfield.ConOTS_local_photons[i] = 0.;
		rfield.ConOTS_local_OTS_rate[i] = 0.;
	}

#	ifdef DEBUG_FUN
	fputs( " <->RT_OTS_Zero()\n", debug_fp );
#	endif
	return;
}

/* =================================================================== */

/*RT_OTS_ChkSum sanity check confirms summed continua reflect contents of individuals */
void RT_OTS_ChkSum(long int ipPnt)
{
	static long int nInsane=0;
	long int i;
	float phisig;

#	ifdef DEBUG_FUN
	fputs( "<+>RT_OTS_ChkSum()\n", debug_fp );
#	endif

	/* this entire sub is a sanity check
	 *begin sanity check */
	for( i=0; i < rfield.nflux; i++ )
	{
		phisig = rfield.otscon[i] + rfield.otslin[i] + rfield.ConInterOut[i]*rfield.lgOutOnly + 
		  rfield.outlin[i]+
		  rfield.ConOTS_local_OTS_rate[i];
		if( phisig > 0. )
		{
			if( fabs(rfield.SummedDif[i]/phisig-1.) > 1e-3 )
			{
				insane();
				++nInsane;
				fprintf( ioQQQ, " RT_OTS_ChkSum insane SummedDif at energy%12.5e error=%10.2e i=%4ld\n", 
				  rfield.anu[i], rfield.SummedDif[i]/phisig - 1., i );
				fprintf( ioQQQ, " SummedDif, sum are%11.4e%11.4e\n", 
				  rfield.SummedDif[i], phisig );
				fprintf( ioQQQ, " otscon otslin ConInterOut outlin are%11.4e%11.4e%11.4e%11.4e\n", 
				  rfield.otscon[i], rfield.otslin[i], rfield.ConInterOut[i], 
				  rfield.outlin[i] );
				fprintf( ioQQQ, " line continuum here are %4.4s %4.4s\n", 
				  rfield.chLineLabel[i], rfield.chContLabel[i] );
			}
		}
		phisig += rfield.flux[i];
		if( phisig > 0. )
		{
			if( fabs(rfield.SummedCon[i]/phisig-1.) > 1e-3 )
			{
				insane();
				++nInsane;
				fprintf( ioQQQ, " RT_OTS_ChkSum%3ld, insane SummedCon at energy%12.5e error=%10.2e i=%4ld\n", 
				  ipPnt, rfield.anu[i], rfield.SummedCon[i]/phisig - 
				  1., i );
				fprintf( ioQQQ, " SummedCon, sum are%11.4e%11.4e\n", 
				  rfield.SummedCon[i], phisig );
				fprintf( ioQQQ, " otscon otslin ConInterOut outlin flux are%11.4e%11.4e%11.4e%11.4e%11.4e\n", 
				  rfield.otscon[i], rfield.otslin[i], rfield.ConInterOut[i], 
				  rfield.outlin[i], rfield.flux[i] );
				fprintf( ioQQQ, " line continuum here are %4.4s %4.4s\n", 
				  rfield.chLineLabel[i], rfield.chContLabel[i]
				   );
			}
		}
	}

	if( nInsane > 10 )
	{
		fprintf( ioQQQ, " RT_OTS_ChkSum too much insanity to continue.\n");
		ShowMe();
		puts( "[Stop in RT_OTS_ChkSum]" );
		cdEXIT(1);
	}

#	ifdef DEBUG_FUN
	fputs( " <->RT_OTS_ChkSum()\n", debug_fp );
#	endif
	return;
}

/* =================================================================== */

/*RT_OTS_PrtRate print continuum and line ots rates when trace ots is on */
void RT_OTS_PrtRate(
	  /* weakest rate to print */
	  double weak ,
	  /* flag, 'c' continuum, 'l' line, 'b' both */
	  int chFlag )
{
	long int i;

#	ifdef DEBUG_FUN
	fputs( "<+>RT_OTS_PrtRate()\n", debug_fp );
#	endif

	/* arg must be one of these three */
	assert( chFlag=='l' || chFlag=='c' || chFlag=='b' );

	/* 
	 * both printouts have cell number (on C array scale)
	 * energy in ryd
	 * the actual value of the ots rate 
	 * the ots rate relative to the continuum at that energy 
	 * rate times opacity 
	 * all are only printed if greater than weak 
	 */

	/*===================================================================*/
	/* first print ots continua                                          */
	/*===================================================================*/
	if( chFlag == 'c' || chFlag == 'b' )
	{
		fprintf( ioQQQ, "     OTSCON array, ANU, OTSCON, OTS/INCID OTS*opac limit=%.2e\n",weak );

		for( i=0; i < rfield.nupper; i++ )
		{
			if( rfield.otscon[i]*opac.opac[i] > weak )
			{
				fprintf( ioQQQ, "     %4ld%12.4e%12.4e%12.4e%12.4e %s \n", 
				  i, 
				  rfield.anu[i], 
				  rfield.otscon[i], 
				  rfield.otscon[i]/MAX2(1e-30,rfield.flux[i]),
				  rfield.otscon[i]*opac.opac[i], 
				  rfield.chContLabel[i]);

			}
		}
	}

	/*===================================================================*/
	/* second print ots line rates                                       */
	/*===================================================================*/
	if( chFlag == 'l' || chFlag == 'b' )
	{
		fprintf( ioQQQ, "     OTSLIN array, ANU, OTSLIN, OTS/INCID OTS*opac limit=%.2e\n",weak );

		for( i=0; i < rfield.nupper; i++ )
		{
			if( rfield.otslin[i]*opac.opac[i] > weak )
			{
				fprintf( ioQQQ, "     %4ld%12.4e%12.4e%12.4e%12.4e %s \n", 
				  i, 
				  rfield.anu[i], 
				  rfield.otslin[i], 
				  rfield.otslin[i]/MAX2(1e-30,rfield.flux[i]),
				  rfield.otslin[i]*opac.opac[i],
				  rfield.chLineLabel[i] );
			}
		}
	}

#	ifdef DEBUG_FUN
	fputs( " <->RT_OTS_PrtRate()\n", debug_fp );
#	endif
	return;
}
